import { Buff } from '@cmdcode/buff';
import PSBTSchema from '../schema.js';
import { consume_keypairs, get_io_counts } from './keypair.js';
import CONST from '../const.js';
export function decode_psbt(psbthex) {
    const stream = Buff.hex(psbthex).stream;
    const magic = stream.read(5).hex;
    if (magic !== CONST.PSBT_MAGIC_BYTES) {
        throw new Error('invalid magic for psbt: ' + magic);
    }
    const global = consume_keypairs(stream, CONST.PSBT_GLOBAL_TYPE_MAP);
    const [vin_count, out_count] = get_io_counts(global);
    const inputs = [];
    for (let i = 0; i < vin_count; i++) {
        const pairs = consume_keypairs(stream, CONST.PSBT_INPUT_TYPE_MAP);
        inputs.push(pairs);
    }
    const outputs = [];
    for (let i = 0; i < out_count; i++) {
        const pairs = consume_keypairs(stream, CONST.PSBT_OUTPUT_TYPE_MAP);
        outputs.push(pairs);
    }
    if (stream.size !== 0) {
        throw new Error('stream has excess bytes: ' + new Buff(stream.data).hex);
    }
    return PSBTSchema.data.parse({ global, inputs, outputs });
}
